package com.testcompany.testapp.gui;

import java.awt.Toolkit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

public class RegexConstrainedDocument extends PlainDocument {

	Pattern pattern;
	Matcher matcher;

	public RegexConstrainedDocument () { super(); } 
	public RegexConstrainedDocument (AbstractDocument.Content c) { super(c); }	
	public RegexConstrainedDocument (AbstractDocument.Content c, String p) {
		super (c);
		setPatternByString (p);
	}
	public RegexConstrainedDocument (String p) {
		super();
		setPatternByString (p);
	}

	public void setPatternByString (String p) {
		Pattern pattern = Pattern.compile (p);
		// checks the document against the new pattern
		// and removes the content if it no longer matches
		try {
			matcher = pattern.matcher (getText(0, getLength())); 
			if (! matcher.matches()) {
				remove (0, getLength());
			}
		} catch (BadLocationException ble) {
			ble.printStackTrace(); // impossible?
		}
	}

	public Pattern getPattern() { return pattern; }

	public void insertString (int offs, String s, AttributeSet a)
		throws BadLocationException {
		// consider whether this insert will match
		String proposedInsert =
			getText (0, offs) +
			s +
			getText (offs, getLength() - offs);
        if (matcher != null) {
			matcher.reset (proposedInsert);
			if (! matcher.matches()) {
				Toolkit.getDefaultToolkit().beep();
				return;
			}
		}
		super.insertString (offs, s, a);
	}
}
