package com.testcompany.testapp.matrixmultiply;

import java.io.OutputStream;
import java.io.PrintStream;

public class Matrix
{
	/* Member Variables */
	private int nCols, nRows;
	private int[][] cells;
	
	
	
	/* Constructors & Initializers */
	public Matrix(int[][] matrix)
	{       
		nCols = matrix.length;
		nRows = matrix[0].length;
		cells = new int[nRows][nCols];
		
		for(int row = 0; row < nRows; row++)
			for(int col = 0; col < nCols; col++)
			{
				cells[row][col] = matrix[row][col];
			}
	}
	
	public Matrix(int size)
	{
		this(size, size);
	}
	
	public Matrix(int nRows, int nCols)
	{
		this.nRows = nRows;
		this.nCols = nCols;
		cells = new int[nRows][nCols];
		for(int i = 0; i < Math.min(nRows, nCols); i++)
			cells[i][i] = 1;
	}
	
	/* Public Methods */
	public void printblock(OutputStream stream)
	{
		PrintStream out = new PrintStream(stream);
		for(int row = 0; row < nRows; row++)
		{
			for(int col = 0; col < nCols; col++)
			{
				out.print(cells[row][col]);
			}
			out.println();
		}
	}
	
	public int getRowCount() { return nRows; }
	public int getColumnCount() { return nCols; }
	
	public boolean isMultipliable(Matrix other)
	{
		if(!this.isSquare() || !other.isSquare()) 
			return false;
		else if(this.getRowCount() != this.getColumnCount())
			return false;
		return true;
	}
	
	public boolean isSquare()
	{
		return nRows == nCols;
	}
	
	public int[][] toIntArray()
	{
		return cells;
	}

	public static Matrix multiply(Matrix a, Matrix b)
	{
		MatrixMultiplyAlg algorithm = new MatrixMultiplyAlg();
		return algorithm.multiply(a, b);
	}
	
	public static Matrix getRandom(int size, int min, int max)
	{
		return getRandom(size, size, min, max);
	}
	
	public static Matrix getRandom(int rows, int cols, int min, int max)
	{
		Matrix result = new Matrix(rows, cols);
		for(int row = 0; row < rows; row++)
			for(int col = 0; col < cols; col++)
			{
				int value = (int)((max - min) * Math.random());
				value += min;
				result.cells[row][col] = value; 
			}
		return result;
	}
}
