package com.testcompany.testapp.matrixmultiply;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;

import com.testcompany.testapp.gui.RegexConstrainedDocument;

public class MatrixPanel extends JPanel
{
	private final int MAX_SIZE = 10;
	private int size = 3;
	private int txtSize = 28;
	
	private GridLayout myLayout;
	
	private JTextField[][] txtCells;

	/* Constructors & Initializers */
	public MatrixPanel()
	{
		super();
		setSize(100, 100);
		setMinimumSize(new Dimension(100, 100));
		this.setBorder(new EmptyBorder(5,5,5,5));
		updateSize(size);
		//
	}
	
	/* Public Methods */
	public void setMatrixSize(int newSize)
	{
		if(newSize <2 || newSize > MAX_SIZE)
			return;
		else
			updateSize(newSize);
	}
	public int getMatrixSize()
	{
		return size;
	}
	
	public int[][] toIntArray()
	{
		int[][] result = new int[size][size];
		for(int row = 0; row < size; row++)
			for(int col = 0; col < size; col++)
			{
				String value = txtCells[row][col].getText();
				try { result[row][col] = Integer.parseInt(value); }
				catch(Throwable t) { result[row][col] = Integer.MAX_VALUE; }
			}
		return result;
	}
	
	public void setMatrix(int[][] newMatrix)
	{
		updateSize(newMatrix.length);
		for(int row = 0; row < size; row++)
			for(int col = 0; col < size; col++)
				txtCells[row][col].setText("" + newMatrix[row][col]);
	}
	
	public void setMatrix(Matrix newMatrix)
	{
		setMatrix(newMatrix.toIntArray());
	}
	
	@Override
	public Dimension getPreferredSize()
	{
		int gapSize = 5;
		int resultSize = size * txtSize;
		resultSize += (size * gapSize) + gapSize;
		return new Dimension(resultSize, resultSize);
	}
	
	@Override
	public void setEnabled(boolean enabled)
	{
		setEditable(enabled);
	}
	
	public void setEditable(boolean editable)
	{
		for(int row = 0; row < size; row++)
			for(int col = 0; col < size; col++)
				txtCells[row][col].setEditable(editable);
	}
	
	public void setCellSize(int newSize)
	{
		txtSize = newSize;
		this.revalidate();
	}
	
	public int getCellSize()
	{
		return txtSize;
	}
	
	/* Private Methods */
	private void updateSize(int newSize)
	{
		if(size == newSize) return;
		
		// Set new size
		size = newSize;
		
		// Discard old components
		Component[] components = this.getComponents();
		for(Component child : components)
		{
			child.setVisible(false);
			this.remove(child);
		}
		
		// Make new layout
		myLayout = new GridLayout(size, size);
		myLayout.setVgap(5);
		myLayout.setHgap(5);
		setLayout(myLayout);
		
		// Make new array
		txtCells = new JTextField[size][size];
		
		for(int row = 0; row < size; row++)
			for(int col = 0; col < size; col++)
			{
				JTextField txt = new JTextField();
				txt.setBorder(new BevelBorder(BevelBorder.LOWERED));
				txt.setHorizontalAlignment(JTextField.CENTER);
				txt.setAlignmentY(JTextField.CENTER_ALIGNMENT);
				txt.setDocument(new RegexConstrainedDocument("(-+)?[0-9]{1,}"));
				txt.setText("0");
				txtCells[row][col] = txt;
				this.add(txt);
			}
		this.setVisible(true);
		this.revalidate();
	}
}
