/* MaxSum.java   8/13/03
 * 
 * Copyright (c) 2003 Rod Howell.
 * 
 * This program compares the running times of 5 algorithms for finding the
 * maximum subsequence sum of an array of integers.  
 */

package com.testcompany.testapp.maxsum;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

/**
 * A class containing a driver for comparing algorithms for finding the maximum
 * subsequence sum of an array of integers. The driver can be executed by a java
 * virtual machine via the command
 * 
 * java MaxSum file alg
 * 
 * where file is the name of an input file and alg is the number of the
 * algorithm, 1-5. The input file must consist of a sequence of integers, one
 * per line. The first line must be a nonnegative integer giving the number of
 * lines that follow; these lines constitute the input for the maximum
 * subsequence sum algorithms.
 * 
 * Upon termination, the program prints a message indicating the maximum
 * subsequence sum and the amount of time required to compute it. The time does
 * not include reading the input.
 */
public class MaxSum
{
	private List<Integer> myIntegers;
	private JProgressBar myProgressBar;
	private JLabel myResult;
	private JLabel myElapsedTime;
	private static MaxSum instance;
	
	private MaxSum() { }
	
	public static MaxSum getInstance()
	{
		if(instance == null)
			instance = new MaxSum();
		return instance;
	}

	public void runAlgorithm()
	{
		// Get Integers if this has not already been done
		if(myIntegers == null) randomize(1000);
		
		MaxSumAlg algorithm = new MaxSumAlg(myIntegers);
		new Thread(algorithm).start();
	}
	
	public void randomize(int numberOfInts)
	{
		myIntegers = new ArrayList<Integer>();
		Random randGenerator = new Random(System.currentTimeMillis());
		for(int i = 0; i < numberOfInts; i++)
			myIntegers.add(randGenerator.nextInt());
	}
	
	public int getNumberOfInts()
	{
		return myIntegers == null ? -1 : myIntegers.size();
	}
	
	public void setProgressBar(JProgressBar progress)
	{
		myProgressBar = progress;
		myProgressBar.setMinimum(0);
		myProgressBar.setMaximum(100);
	}
	
	public void setResultControls(JLabel result, JLabel elapsed)
	{
		myResult = result;
		myElapsedTime = elapsed;
	}
	
	public void finishTask()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				myProgressBar.setIndeterminate(false);
			}
		});
	}
	
	public void startTask()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				myProgressBar.setIndeterminate(true);
			}
		});
	}
	
	public void setResult(final int maxSum, final long elapsedTime)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				myResult.setText("" + maxSum);
				myElapsedTime.setText(elapsedTime + "ms");
			}
		});
	}
	
}
